@@ -1,5 +1,5 @@
 /**
- * Copyright (c) 2006-2014 LOVE Development Team
+ * Copyright (c) 2006-2016 LOVE Development Team
  *
  * This software is provided 'as-is', without any express or implied
  * warranty.  In no event will the authors be held liable for any damages
@@ -132,6 +132,15 @@ void Audio::setVelocity(float *)
 {
 }
 
+void Audio::setDopplerScale(float)
+{
+}
+
+float Audio::getDopplerScale() const
+{
+	return 1.0f;
+}
+
 void Audio::record()
 {
 }
