@@ -1,5 +1,5 @@
 /**
- * Copyright (c) 2006-2014 LOVE Development Team
+ * Copyright (c) 2006-2016 LOVE Development Team
  *
  * This software is provided 'as-is', without any express or implied
  * warranty.  In no event will the authors be held liable for any damages
@@ -26,25 +26,25 @@
 
 #include "common/runtime.h"
 
+// C++
+#include <iostream>
+
 namespace love
 {
 namespace audio
 {
 
-static Audio *instance = 0;
+#define instance() (Module::getInstance<Audio>(Module::M_AUDIO))
 
 int w_getSourceCount(lua_State *L)
 {
-	lua_pushinteger(L, instance->getSourceCount());
+	lua_pushinteger(L, instance()->getSourceCount());
 	return 1;
 }
 
 int w_newSource(lua_State *L)
 {
-	if (lua_isstring(L, 1) || luax_istype(L, 1, FILESYSTEM_FILE_T))
-		luax_convobj(L, 1, "filesystem", "newFileData");
-
-	if (luax_istype(L, 1, FILESYSTEM_FILE_DATA_T))
+	if (lua_isstring(L, 1) || luax_istype(L, 1, FILESYSTEM_FILE_ID) || luax_istype(L, 1, FILESYSTEM_FILE_DATA_ID))
 		luax_convobj(L, 1, "sound", "newDecoder");
 
 	Source::Type stype = Source::TYPE_STREAM;
@@ -53,19 +53,22 @@ int w_newSource(lua_State *L)
 	if (stypestr && !Source::getConstant(stypestr, stype))
 		return luaL_error(L, "Invalid source type: %s", stypestr);
 
-	if (stype == Source::TYPE_STATIC && luax_istype(L, 1, SOUND_DECODER_T))
+	if (stype == Source::TYPE_STATIC && luax_istype(L, 1, SOUND_DECODER_ID))
 		luax_convobj(L, 1, "sound", "newSoundData");
 
 	Source *t = 0;
 
-	if (luax_istype(L, 1, SOUND_SOUND_DATA_T))
-		t = instance->newSource(luax_totype<love::sound::SoundData>(L, 1, "SoundData", SOUND_SOUND_DATA_T));
-	else if (luax_istype(L, 1, SOUND_DECODER_T))
-		t = instance->newSource(luax_totype<love::sound::Decoder>(L, 1, "Decoder", SOUND_DECODER_T));
+	luax_catchexcept(L, [&]() {
+		if (luax_istype(L, 1, SOUND_SOUND_DATA_ID))
+			t = instance()->newSource(luax_totype<love::sound::SoundData>(L, 1, SOUND_SOUND_DATA_ID));
+		else if (luax_istype(L, 1, SOUND_DECODER_ID))
+			t = instance()->newSource(luax_totype<love::sound::Decoder>(L, 1, SOUND_DECODER_ID));
+	});
 
 	if (t)
 	{
-		luax_pushtype(L, "Source", AUDIO_SOURCE_T, t);
+		luax_pushtype(L, AUDIO_SOURCE_ID, t);
+		t->release();
 		return 1;
 	}
 	else
@@ -75,7 +78,7 @@ int w_newSource(lua_State *L)
 int w_play(lua_State *L)
 {
 	Source *s = luax_checksource(L, 1);
-	luax_pushboolean(L, instance->play(s));
+	luax_pushboolean(L, instance()->play(s));
 	return 1;
 }
 
@@ -83,7 +86,7 @@ int w_stop(lua_State *L)
 {
 	if (lua_gettop(L) == 0)
 	{
-		instance->stop();
+		instance()->stop();
 	}
 	else
 	{
@@ -97,7 +100,7 @@ int w_pause(lua_State *L)
 {
 	if (lua_gettop(L) == 0)
 	{
-		instance->pause();
+		instance()->pause();
 	}
 	else
 	{
@@ -112,7 +115,7 @@ int w_resume(lua_State *L)
 {
 	if (lua_gettop(L) == 0)
 	{
-		instance->resume();
+		instance()->resume();
 	}
 	else
 	{
@@ -126,7 +129,7 @@ int w_rewind(lua_State *L)
 {
 	if (lua_gettop(L) == 0)
 	{
-		instance->rewind();
+		instance()->rewind();
 	}
 	else
 	{
@@ -139,13 +142,13 @@ int w_rewind(lua_State *L)
 int w_setVolume(lua_State *L)
 {
 	float v = (float)luaL_checknumber(L, 1);
-	instance->setVolume(v);
+	instance()->setVolume(v);
 	return 0;
 }
 
 int w_getVolume(lua_State *L)
 {
-	lua_pushnumber(L, instance->getVolume());
+	lua_pushnumber(L, instance()->getVolume());
 	return 1;
 }
 
@@ -155,14 +158,14 @@ int w_setPosition(lua_State *L)
 	v[0] = (float)luaL_checknumber(L, 1);
 	v[1] = (float)luaL_checknumber(L, 2);
 	v[2] = (float)luaL_optnumber(L, 3, 0);
-	instance->setPosition(v);
+	instance()->setPosition(v);
 	return 0;
 }
 
 int w_getPosition(lua_State *L)
 {
 	float v[3];
-	instance->getPosition(v);
+	instance()->getPosition(v);
 	lua_pushnumber(L, v[0]);
 	lua_pushnumber(L, v[1]);
 	lua_pushnumber(L, v[2]);
@@ -178,14 +181,14 @@ int w_setOrientation(lua_State *L)
 	v[3] = (float)luaL_checknumber(L, 4);
 	v[4] = (float)luaL_checknumber(L, 5);
 	v[5] = (float)luaL_checknumber(L, 6);
-	instance->setOrientation(v);
+	instance()->setOrientation(v);
 	return 0;
 }
 
 int w_getOrientation(lua_State *L)
 {
 	float v[6];
-	instance->getOrientation(v);
+	instance()->getOrientation(v);
 	lua_pushnumber(L, v[0]);
 	lua_pushnumber(L, v[1]);
 	lua_pushnumber(L, v[2]);
@@ -201,33 +204,48 @@ int w_setVelocity(lua_State *L)
 	v[0] = (float)luaL_checknumber(L, 1);
 	v[1] = (float)luaL_checknumber(L, 2);
 	v[2] = (float)luaL_optnumber(L, 3, 0);
-	instance->setVelocity(v);
+	instance()->setVelocity(v);
 	return 0;
 }
 
 int w_getVelocity(lua_State *L)
 {
 	float v[3];
-	instance->getVelocity(v);
+	instance()->getVelocity(v);
 	lua_pushnumber(L, v[0]);
 	lua_pushnumber(L, v[1]);
 	lua_pushnumber(L, v[2]);
 	return 3;
 }
 
+int w_setDopplerScale(lua_State *L)
+{
+	instance()->setDopplerScale(luax_checkfloat(L, 1));
+	return 0;
+}
+
+int w_getDopplerScale(lua_State *L)
+{
+	lua_pushnumber(L, instance()->getDopplerScale());
+	return 1;
+}
+
 int w_record(lua_State *)
 {
-	instance->record();
+	instance()->record();
 	return 0;
 }
 
 int w_getRecordedData(lua_State *L)
 {
-	love::sound::SoundData *sd = instance->getRecordedData();
+	love::sound::SoundData *sd = instance()->getRecordedData();
 	if (!sd)
 		lua_pushnil(L);
 	else
-		luax_pushtype(L, "SoundData", SOUND_SOUND_DATA_T, sd);
+	{
+		luax_pushtype(L, SOUND_SOUND_DATA_ID, sd);
+		sd->release();
+	}
 	return 1;
 }
 
@@ -235,18 +253,23 @@ int w_stopRecording(lua_State *L)
 {
 	if (luax_optboolean(L, 1, true))
 	{
-		love::sound::SoundData *sd = instance->stopRecording(true);
-		if (!sd) lua_pushnil(L);
-		else luax_pushtype(L, "SoundData", SOUND_SOUND_DATA_T, sd);
+		love::sound::SoundData *sd = instance()->stopRecording(true);
+		if (!sd)
+			lua_pushnil(L);
+		else
+		{
+			luax_pushtype(L, SOUND_SOUND_DATA_ID, sd);
+			sd->release();
+		}
 		return 1;
 	}
-	instance->stopRecording(false);
+	instance()->stopRecording(false);
 	return 0;
 }
 
 int w_canRecord(lua_State *L)
 {
-	luax_pushboolean(L, instance->canRecord());
+	luax_pushboolean(L, instance()->canRecord());
 	return 1;
 }
 
@@ -256,13 +279,13 @@ int w_setDistanceModel(lua_State *L)
 	Audio::DistanceModel distanceModel;
 	if (!Audio::getConstant(modelStr, distanceModel))
 		return luaL_error(L, "Invalid distance model: %s", modelStr);
-	instance->setDistanceModel(distanceModel);
+	instance()->setDistanceModel(distanceModel);
 	return 0;
 }
 
 int w_getDistanceModel(lua_State *L)
 {
-	Audio::DistanceModel distanceModel = instance->getDistanceModel();
+	Audio::DistanceModel distanceModel = instance()->getDistanceModel();
 	const char *modelStr;
 	if (!Audio::getConstant(distanceModel, modelStr))
 		return 0;
@@ -288,6 +311,8 @@ static const luaL_Reg functions[] =
 	{ "getOrientation", w_getOrientation },
 	{ "setVelocity", w_setVelocity },
 	{ "getVelocity", w_getVelocity },
+	{ "setDopplerScale", w_setDopplerScale },
+	{ "getDopplerScale", w_getDopplerScale },
 	/*{ "record", w_record },
 	{ "getRecordedData", w_getRecordedData },
 	{ "stopRecording", w_stopRecording },*/
@@ -304,8 +329,10 @@ static const lua_CFunction types[] =
 
 extern "C" int luaopen_love_audio(lua_State *L)
 {
+	Audio *instance = instance();
+
 #ifdef LOVE_ENABLE_AUDIO_OPENAL
-	if (instance == 0)
+	if (instance == nullptr)
 	{
 		// Try OpenAL first.
 		try
@@ -322,7 +349,7 @@ extern "C" int luaopen_love_audio(lua_State *L)
 #endif
 
 #ifdef LOVE_ENABLE_AUDIO_NULL
-	if (instance == 0)
+	if (instance == nullptr)
 	{
 		// Fall back to nullaudio.
 		try
@@ -336,13 +363,13 @@ extern "C" int luaopen_love_audio(lua_State *L)
 	}
 #endif
 
-	if (instance == 0)
+	if (instance == nullptr)
 		return luaL_error(L, "Could not open any audio module.");
 
 	WrappedModule w;
 	w.module = instance;
 	w.name = "audio";
-	w.flags = MODULE_T;
+	w.type = MODULE_ID;
 	w.functions = functions;
 	w.types = types;
 
