@@ -1,5 +1,5 @@
 /**
- * Copyright (c) 2006-2015 LOVE Development Team
+ * Copyright (c) 2006-2016 LOVE Development Team
  *
  * This software is provided 'as-is', without any express or implied
  * warranty.  In no event will the authors be held liable for any damages
@@ -49,10 +49,10 @@ public:
 	 **/
 	enum Mode
 	{
-		CLOSED,
-		READ,
-		WRITE,
-		APPEND,
+		MODE_CLOSED,
+		MODE_READ,
+		MODE_WRITE,
+		MODE_APPEND,
 		MODE_MAX_ENUM
 	};
 
@@ -77,7 +77,7 @@ public:
 	/**
 	 * Opens the file in a certain mode.
 	 *
-	 * @param mode READ, WRITE, APPEND.
+	 * @param mode MODE_READ, MODE_WRITE, MODE_APPEND.
 	 * @return True if successful, false otherwise.
 	 **/
 	virtual bool open(Mode mode) = 0;
@@ -107,7 +107,7 @@ public:
 	 * @param size The number of bytes to attempt reading, or -1 for EOF.
 	 * @return A newly allocated Data object.
 	 **/
-	virtual FileData *read(int64 size = ALL) = 0;
+	virtual FileData *read(int64 size = ALL);
 
 	/**
 	 * Reads data into the destination buffer.
@@ -134,7 +134,7 @@ public:
 	 * @param size The number of bytes to attempt writing, or -1 for everything.
 	 * @return True of success, false otherwise.
 	 **/
-	virtual bool write(const Data *data, int64 size = ALL) = 0;
+	virtual bool write(const Data *data, int64 size = ALL);
 
 	/**
 	 * Flushes the currently buffered file data to disk. Only applicable in
@@ -147,7 +147,7 @@ public:
 	 *
 	 * @return True if EOF, false otherwise.
 	 **/
-	virtual bool eof() = 0;
+	virtual bool isEOF() = 0;
 
 	/**
 	 * Gets the current position in the File.
@@ -192,13 +192,13 @@ public:
 	 * Gets the filename for this File, or empty string if none.
 	 * @return The filename for this File.
 	 **/
-	virtual std::string getFilename() const = 0;
+	virtual const std::string &getFilename() const = 0;
 
 	/**
 	 * Gets the file extension for this File, or empty string if none.
 	 * @return The file extension for this File (without the dot).
 	 **/
-	virtual std::string getExtension() const = 0;
+	virtual std::string getExtension() const;
 
 	static bool getConstant(const char *in, Mode &out);
 	static bool getConstant(Mode in, const char *&out);
