@@ -1,5 +1,5 @@
 /**
- * Copyright (c) 2006-2015 LOVE Development Team
+ * Copyright (c) 2006-2016 LOVE Development Team
  *
  * This software is provided 'as-is', without any express or implied
  * warranty.  In no event will the authors be held liable for any damages
@@ -21,13 +21,15 @@
 #ifndef LOVE_GRAPHICS_OPENGL_CANVAS_H
 #define LOVE_GRAPHICS_OPENGL_CANVAS_H
 
+#include "common/config.h"
 #include "graphics/Color.h"
 #include "image/Image.h"
 #include "image/ImageData.h"
 #include "common/Matrix.h"
 #include "common/StringMap.h"
 #include "common/int.h"
-#include "Texture.h"
+#include "graphics/Texture.h"
+#include "graphics/Volatile.h"
 #include "OpenGL.h"
 
 namespace love
@@ -37,24 +39,30 @@ namespace graphics
 namespace opengl
 {
 
-class Canvas : public Texture
+class Canvas : public Texture, public Volatile
 {
 public:
 
 	// Different Canvas render target formats.
 	enum Format
 	{
-		FORMAT_NORMAL,   // Usually RGBA8 or a similar fallback. Always supported.
+		FORMAT_NORMAL,   // Usually SRGB, RGBA8 or a similar fallback. Always supported.
 		FORMAT_HDR,      // Usually RGBA16F. Not always supported.
-		FORMAT_RGBA8,    // RGBA with 8 bits per component.
-		FORMAT_RGBA4,    // RGBA with 4 bits per component.
-		FORMAT_RGB5A1,   // RGB with 5 bits per component, and A with 1 bit.
+		FORMAT_RGBA4,    // RGBA with 4 bits per channel.
+		FORMAT_RGB5A1,   // RGB with 5 bits per channel, and A with 1 bit.
 		FORMAT_RGB565,   // RGB with 5, 6, and 5 bits each, respectively.
+		FORMAT_R8,       // Single (red) 8-bit channel.
+		FORMAT_RG8,      // Two-channel (red and green) with 8 bits per channel.
+		FORMAT_RGBA8,    // RGBA with 8 bits per channel.
 		FORMAT_RGB10A2,  // RGB with 10 bits each, and A with 2 bits.
-		FORMAT_RG11B10F, // Floating point [0, +inf]. RG with 11 FP bits each, and B with 10 FP bits.
-		FORMAT_RGBA16F,  // Floating point [-inf, +inf]. RGBA with 16 FP bits per component.
-		FORMAT_RGBA32F,  // Floating point [-inf, +inf]. RGBA with 32 FP bits per component.
-		FORMAT_SRGB,     // sRGB with 8 bits per component, plus 8 bit linear A.
+		FORMAT_RG11B10F, // Floating point [0, +65024]. RG with 11 FP bits each, and B with 10 FP bits.
+		FORMAT_R16F,     // Floating point [-65504, +65504]. R with 16 FP bits.
+		FORMAT_RG16F,    // Floating point [-65504, +65504]. RG with 16 FP bits per channel.
+		FORMAT_RGBA16F,  // Floating point [-65504, +65504]. RGBA with 16 FP bits per channel.
+		FORMAT_R32F,     // Floating point [-65504, +65504]. R with 32 FP bits.
+		FORMAT_RG32F,    // Floating point [-65504, +65504]. RG with 32 FP bits per channel.
+		FORMAT_RGBA32F,  // Floating point [-65504, +65504]. RGBA with 32 FP bits per channel.
+		FORMAT_SRGB,     // sRGB with 8 bits per channel, plus 8 bit linear A.
 		FORMAT_MAX_ENUM
 	};
 
@@ -71,9 +79,8 @@ public:
 	// Implements Texture.
 	virtual void drawq(Quad *quad, float x, float y, float angle, float sx, float sy, float ox, float oy, float kx, float ky);
 	virtual void setFilter(const Texture::Filter &f);
-	virtual void setWrap(const Texture::Wrap &w);
-	virtual GLuint getGLTexture() const;
-	virtual void predraw();
+	virtual bool setWrap(const Texture::Wrap &w);
+	virtual const void *getHandle() const;
 
 	/**
 	 * @param canvases A list of other canvases to temporarily attach to this one,
@@ -83,16 +90,12 @@ public:
 	void startGrab();
 	void stopGrab(bool switchingToOtherCanvas = false);
 
-	void clear(Color c);
-
 	/**
 	 * Create and attach a stencil buffer to this Canvas' framebuffer, if necessary.
 	 **/
 	bool checkCreateStencil();
 
-	love::image::ImageData *getImageData(love::image::Image *image);
-
-	void getPixel(unsigned char* pixel_rgba, int x, int y);
+	love::image::ImageData *newImageData(love::image::Image *image, int x, int y, int w, int h);
 
 	inline const std::vector<Canvas *> &getAttachedCanvases() const
 	{
@@ -111,13 +114,12 @@ public:
 
 	inline int getMSAA() const
 	{
-		return msaa_samples;
+		return actual_samples;
 	}
 
-	bool resolveMSAA();
-
+	static Format getSizedFormat(Format format);
 	static bool isSupported();
-	static bool isMultiCanvasSupported();
+	static bool isMultiFormatMultiCanvasSupported();
 	static bool isFormatSupported(Format format);
 
 	static Canvas *current;
@@ -128,7 +130,6 @@ public:
 	// Whether the main screen should have linear -> sRGB conversions enabled.
 	static bool screenHasSRGB;
 
-	static int switchCount;
 	static int canvasCount;
 
 	static bool getConstant(const char *in, Format &out);
@@ -136,9 +137,13 @@ public:
 
 private:
 
+	void setupGrab();
+
 	bool createMSAAFBO(GLenum internalformat);
+	bool resolveMSAA(bool restoreprev);
+
+	void drawv(const Matrix4 &t, const Vertex *v);
 
-	static Format getSizedFormat(Format format);
 	static void convertFormat(Format format, GLenum &internalformat, GLenum &externalformat, GLenum &type);
 	static size_t getFormatBitsPerPixel(Format format);
 
@@ -155,14 +160,11 @@ private:
 
 	std::vector<Canvas *> attachedCanvases;
 
-	int msaa_samples;
-	bool msaa_dirty;
+	int requested_samples;
+	int actual_samples;
 
 	size_t texture_memory;
 
-	void setupGrab();
-	void drawv(const Matrix &t, const Vertex *v);
-
 	static bool supportedFormats[FORMAT_MAX_ENUM];
 	static bool checkedFormats[FORMAT_MAX_ENUM];
 
