@@ -1,5 +1,5 @@
 /**
- * Copyright (c) 2006-2014 LOVE Development Team
+ * Copyright (c) 2006-2016 LOVE Development Team
  *
  * This software is provided 'as-is', without any express or implied
  * warranty.  In no event will the authors be held liable for any damages
@@ -29,7 +29,7 @@ namespace love
 namespace joystick
 {
 
-float Joystick::clampval(float x) const
+float Joystick::clampval(float x)
 {
 	if (fabsf(x) < 0.01)
 		return 0.0f;
