@@ -1,5 +1,5 @@
 /**
- * Copyright (c) 2006-2014 LOVE Development Team
+ * Copyright (c) 2006-2016 LOVE Development Team
  *
  * This software is provided 'as-is', without any express or implied
  * warranty.  In no event will the authors be held liable for any damages
@@ -48,7 +48,7 @@ class Fixture;
 struct bodyudata
 {
 	// Reference to arbitrary data.
-	Reference *ref;
+	Reference *ref = nullptr;
 };
 
 /**
@@ -393,6 +393,18 @@ public:
 	 **/
 	int getFixtureList(lua_State *L) const;
 
+	/**
+	 * Get an array of all Joints attached to this Body.
+	 **/
+	int getJointList(lua_State *L) const;
+
+	/**
+	 * Get an array of all active Contacts attached to this Body.
+	 * This list changes during World:update and you may miss some collisions
+	 * if you don't use the collision callbacks.
+	 **/
+	int getContactList(lua_State *L) const;
+
 	/**
 	 * Destroy this body.
 	 **/
@@ -423,12 +435,8 @@ private:
 	 **/
 	int pushVector(lua_State *L, const b2Vec2 &v);
 
-	// We need a shared_ptr to the parent World,
-	// because World can not be destroyed as long as
-	// bodies exists in it.
-	//
-	// This ensures that a World only can be destroyed
-	// once all bodies have been destroyed too.
+	// FIXME: This should be a weak reference, rather than being completely
+	// unowned?
 	World *world;
 
 	bodyudata *udata;
