@@ -1,5 +1,5 @@
 /**
- * Copyright (c) 2006-2014 LOVE Development Team
+ * Copyright (c) 2006-2016 LOVE Development Team
  *
  * This software is provided 'as-is', without any express or implied
  * warranty.  In no event will the authors be held liable for any damages
@@ -47,6 +47,14 @@ public:
 
 	virtual ~EdgeShape();
 
+	void setNextVertex(float x, float y);
+	void setNextVertex();
+	bool getNextVertex(float &x, float &y) const;
+
+	void setPreviousVertex(float x, float y);
+	void setPreviousVertex();
+	bool getPreviousVertex(float &x, float &y) const;
+
 	/**
 	 * Returns the transformed points of the edge shape.
 	 * This function is useful for debug drawing and such.
