@@ -1,5 +1,5 @@
 /**
- * Copyright (c) 2006-2014 LOVE Development Team
+ * Copyright (c) 2006-2016 LOVE Development Team
  *
  * This software is provided 'as-is', without any express or implied
  * warranty.  In no event will the authors be held liable for any damages
@@ -64,6 +64,9 @@ public:
 	 **/
 	float getRatio() const;
 
+	Joint *getJointA() const;
+	Joint *getJointB() const;
+
 private:
 	// The Box2D GearJoint object.
 	b2GearJoint *joint;
