@@ -1,5 +1,5 @@
 /**
- * Copyright (c) 2006-2014 LOVE Development Team
+ * Copyright (c) 2006-2016 LOVE Development Team
  *
  * This software is provided 'as-is', without any express or implied
  * warranty.  In no event will the authors be held liable for any damages
@@ -19,6 +19,7 @@
  **/
 
 #include "wrap_Contact.h"
+#include "Fixture.h"
 
 namespace love
 {
@@ -29,7 +30,7 @@ namespace box2d
 
 Contact *luax_checkcontact(lua_State *L, int idx)
 {
-	Contact *c = luax_checktype<Contact>(L, idx, "Contact", PHYSICS_CONTACT_T);
+	Contact *c = luax_checktype<Contact>(L, idx, PHYSICS_CONTACT_ID);
 	if (!c->isValid())
 		luaL_error(L, "Attempt to use destroyed contact.");
 	return c;
@@ -138,28 +139,49 @@ int w_Contact_getChildren(lua_State *L)
 	return 2;
 }
 
+int w_Contact_getFixtures(lua_State *L)
+{
+	Contact *t = luax_checkcontact(L, 1);
+	Fixture *a = nullptr;
+	Fixture *b = nullptr;
+	luax_catchexcept(L, [&](){ t->getFixtures(a, b); });
+
+	luax_pushtype(L, PHYSICS_FIXTURE_ID, a);
+	luax_pushtype(L, PHYSICS_FIXTURE_ID, b);
+	return 2;
+}
+
+int w_Contact_isDestroyed(lua_State *L)
+{
+	Contact *c = luax_checktype<Contact>(L, 1, PHYSICS_CONTACT_ID);
+	luax_pushboolean(L, !c->isValid());
+	return 1;
+}
+
+static const luaL_Reg w_Contact_functions[] =
+{
+	{ "getPositions", w_Contact_getPositions },
+	{ "getNormal", w_Contact_getNormal },
+	{ "getFriction", w_Contact_getFriction },
+	{ "getRestitution", w_Contact_getRestitution },
+	{ "isEnabled", w_Contact_isEnabled },
+	{ "isTouching", w_Contact_isTouching },
+	{ "setFriction", w_Contact_setFriction },
+	{ "setRestitution", w_Contact_setRestitution },
+	{ "setEnabled", w_Contact_setEnabled },
+	{ "resetFriction", w_Contact_resetFriction },
+	{ "resetRestitution", w_Contact_resetRestitution },
+	{ "setTangentSpeed", w_Contact_setTangentSpeed },
+	{ "getTangentSpeed", w_Contact_getTangentSpeed },
+	{ "getChildren", w_Contact_getChildren },
+	{ "getFixtures", w_Contact_getFixtures },
+	{ "isDestroyed", w_Contact_isDestroyed },
+	{ 0, 0 }
+};
+
 extern "C" int luaopen_contact(lua_State *L)
 {
-	static const luaL_Reg functions[] =
-	{
-		{ "getPositions", w_Contact_getPositions },
-		{ "getNormal", w_Contact_getNormal },
-		{ "getFriction", w_Contact_getFriction },
-		{ "getRestitution", w_Contact_getRestitution },
-		{ "isEnabled", w_Contact_isEnabled },
-		{ "isTouching", w_Contact_isTouching },
-		{ "setFriction", w_Contact_setFriction },
-		{ "setRestitution", w_Contact_setRestitution },
-		{ "setEnabled", w_Contact_setEnabled },
-		{ "resetFriction", w_Contact_resetFriction },
-		{ "resetRestitution", w_Contact_resetRestitution },
-		{ "setTangentSpeed", w_Contact_setTangentSpeed },
-		{ "getTangentSpeed", w_Contact_getTangentSpeed },
-		{ "getChildren", w_Contact_getChildren },
-		{ 0, 0 }
-	};
-
-	return luax_register_type(L, "Contact", functions);
+	return luax_register_type(L, PHYSICS_CONTACT_ID, "Contact", w_Contact_functions, nullptr);
 }
 
 } // box2d
