@@ -1,5 +1,5 @@
 /**
- * Copyright (c) 2006-2014 LOVE Development Team
+ * Copyright (c) 2006-2016 LOVE Development Team
  *
  * This software is provided 'as-is', without any express or implied
  * warranty.  In no event will the authors be held liable for any damages
@@ -34,10 +34,6 @@ namespace box2d
 {
 
 PulleyJoint *luax_checkpulleyjoint(lua_State *L, int idx);
-int w_PulleyJoint_getGroundAnchors(lua_State *L);
-int w_PulleyJoint_getLengthA(lua_State *L);
-int w_PulleyJoint_getLengthB(lua_State *L);
-int w_PulleyJoint_getRatio(lua_State *L);
 extern "C" int luaopen_pulleyjoint(lua_State *L);
 
 } // box2d
