@@ -1,5 +1,5 @@
 /**
- * Copyright (c) 2006-2014 LOVE Development Team
+ * Copyright (c) 2006-2016 LOVE Development Team
  *
  * This software is provided 'as-is', without any express or implied
  * warranty.  In no event will the authors be held liable for any damages
@@ -34,17 +34,10 @@ namespace box2d
 {
 
 Shape *luax_checkshape(lua_State *L, int idx);
-int w_Shape_getType(lua_State *L);
-int w_Shape_getRadius(lua_State *L);
-
-int w_Shape_getChildCount(lua_State *L);
-int w_Shape_testPoint(lua_State *L);
-int w_Shape_rayCast(lua_State *L);
-int w_Shape_computeAABB(lua_State *L);
-int w_Shape_computeMass(lua_State *L);
-
 extern "C" int luaopen_shape(lua_State *L);
 
+extern const luaL_Reg w_Shape_functions[];
+
 } // box2d
 } // physics
 } // love
