@@ -1,5 +1,5 @@
 /**
- * Copyright (c) 2006-2014 LOVE Development Team
+ * Copyright (c) 2006-2016 LOVE Development Team
  *
  * This software is provided 'as-is', without any express or implied
  * warranty.  In no event will the authors be held liable for any damages
@@ -49,7 +49,7 @@ public:
 		SETTING_FULLSCREEN,
 		SETTING_FULLSCREEN_TYPE,
 		SETTING_VSYNC,
-		SETTING_FSAA,
+		SETTING_MSAA,
 		SETTING_RESIZABLE,
 		SETTING_MIN_WIDTH,
 		SETTING_MIN_HEIGHT,
@@ -57,43 +57,79 @@ public:
 		SETTING_CENTERED,
 		SETTING_DISPLAY,
 		SETTING_HIGHDPI,
-		SETTING_SRGB,
+		SETTING_REFRESHRATE,
+		SETTING_X,
+		SETTING_Y,
 		SETTING_MAX_ENUM
 	};
 
 	enum FullscreenType
 	{
-		FULLSCREEN_TYPE_NORMAL,
-		FULLSCREEN_TYPE_DESKTOP,
-		FULLSCREEN_TYPE_MAX_ENUM
+		FULLSCREEN_EXCLUSIVE,
+		FULLSCREEN_DESKTOP,
+		FULLSCREEN_MAX_ENUM
+	};
+
+	enum MessageBoxType
+	{
+		MESSAGEBOX_ERROR,
+		MESSAGEBOX_WARNING,
+		MESSAGEBOX_INFO,
+		MESSAGEBOX_MAX_ENUM
 	};
 
 	struct WindowSize
 	{
 		int width;
 		int height;
+
+		bool operator == (const WindowSize &w) const
+		{
+			return w.width == width && w.height == height;
+		}
+	};
+
+	struct MessageBoxData
+	{
+		MessageBoxType type;
+
+		std::string title;
+		std::string message;
+
+		std::vector<std::string> buttons;
+		int enterButtonIndex;
+		int escapeButtonIndex;
+
+		bool attachToWindow;
 	};
 
 	virtual ~Window();
 
+	// Implements Module.
+	virtual ModuleType getModuleType() const { return M_WINDOW; }
+
 	virtual bool setWindow(int width = 800, int height = 600, WindowSettings *settings = nullptr) = 0;
 	virtual void getWindow(int &width, int &height, WindowSettings &settings) = 0;
 
+	virtual void close() = 0;
+
 	virtual bool setFullscreen(bool fullscreen, FullscreenType fstype) = 0;
 	virtual bool setFullscreen(bool fullscreen) = 0;
 
-	virtual bool onWindowResize(int width, int height) = 0;
+	virtual bool onSizeChanged(int width, int height) = 0;
 
 	virtual int getDisplayCount() const = 0;
 
-	virtual std::vector<WindowSize> getFullscreenSizes(int displayindex) const = 0;
+	virtual const char *getDisplayName(int displayindex) const = 0;
 
-	virtual int getWidth() const = 0;
-	virtual int getHeight() const = 0;
+	virtual std::vector<WindowSize> getFullscreenSizes(int displayindex) const = 0;
 
 	virtual void getDesktopDimensions(int displayindex, int &width, int &height) const = 0;
 
-	virtual bool isCreated() const = 0;
+	virtual void setPosition(int x, int y, int displayindex) = 0;
+	virtual void getPosition(int &x, int &y, int &displayindex) = 0;
+
+	virtual bool isOpen() const = 0;
 
 	virtual void setWindowTitle(const std::string &title) = 0;
 	virtual const std::string &getWindowTitle() const = 0;
@@ -101,6 +137,14 @@ public:
 	virtual bool setIcon(love::image::ImageData *imgd) = 0;
 	virtual love::image::ImageData *getIcon() = 0;
 
+	virtual void setDisplaySleepEnabled(bool enable) = 0;
+	virtual bool isDisplaySleepEnabled() const = 0;
+
+	virtual void minimize() = 0;
+	virtual void maximize() = 0;
+
+	virtual bool isMaximized() const = 0;
+
 	// default no-op implementation
 	virtual void swapBuffers();
 
@@ -109,19 +153,28 @@ public:
 
 	virtual bool isVisible() const = 0;
 
-	virtual void setMouseVisible(bool visible) = 0;
-	virtual bool getMouseVisible() const = 0;
-
 	virtual void setMouseGrab(bool grab) = 0;
 	virtual bool isMouseGrabbed() const = 0;
 
+	virtual void getPixelDimensions(int &w, int &h) const = 0;
+	// Note: window-space coordinates are not necessarily the same as
+	// density-independent units (which toPixels and fromPixels use.)
+	virtual void windowToPixelCoords(double *x, double *y) const = 0;
+	virtual void pixelToWindowCoords(double *x, double *y) const = 0;
+
 	virtual double getPixelScale() const = 0;
 
+	virtual double toPixels(double x) const = 0;
+	virtual void toPixels(double wx, double wy, double &px, double &py) const = 0;
+	virtual double fromPixels(double x) const = 0;
+	virtual void fromPixels(double px, double py, double &wx, double &wy) const = 0;
+
 	virtual const void *getHandle() const = 0;
 
-	//virtual static Window *createSingleton() = 0;
-	//virtual static Window *getSingleton() = 0;
-	// No virtual statics, of course, but you are supposed to implement these statics.
+	virtual bool showMessageBox(const std::string &title, const std::string &message, MessageBoxType type, bool attachtowindow) = 0;
+	virtual int showMessageBox(const MessageBoxData &data) = 0;
+
+	virtual void requestAttention(bool continuous) = 0;
 
 	static bool getConstant(const char *in, Setting &out);
 	static bool getConstant(Setting in, const char *&out);
@@ -129,38 +182,40 @@ public:
 	static bool getConstant(const char *in, FullscreenType &out);
 	static bool getConstant(FullscreenType in, const char *&out);
 
-protected:
-
-	static Window *singleton;
+	static bool getConstant(const char *in, MessageBoxType &out);
+	static bool getConstant(MessageBoxType in, const char *&out);
 
 private:
 
 	static StringMap<Setting, SETTING_MAX_ENUM>::Entry settingEntries[];
 	static StringMap<Setting, SETTING_MAX_ENUM> settings;
 
-	static StringMap<FullscreenType, FULLSCREEN_TYPE_MAX_ENUM>::Entry fullscreenTypeEntries[];
-	static StringMap<FullscreenType, FULLSCREEN_TYPE_MAX_ENUM> fullscreenTypes;
+	static StringMap<FullscreenType, FULLSCREEN_MAX_ENUM>::Entry fullscreenTypeEntries[];
+	static StringMap<FullscreenType, FULLSCREEN_MAX_ENUM> fullscreenTypes;
+
+	static StringMap<MessageBoxType, MESSAGEBOX_MAX_ENUM>::Entry messageBoxTypeEntries[];
+	static StringMap<MessageBoxType, MESSAGEBOX_MAX_ENUM> messageBoxTypes;
 
 }; // Window
 
 struct WindowSettings
 {
-	WindowSettings();
-
-	bool fullscreen; // = false
-	Window::FullscreenType fstype; // = FULLSCREEN_TYPE_NORMAL
-	bool vsync; // = true
-	int fsaa; // = 0
-	bool resizable; // = false
-	int minwidth; // = 1
-	int minheight; // = 1
-	bool borderless; // = false
-	bool centered; // = true
-	int display; // = 0
-	bool highdpi; // false
-	bool sRGB; // false
-
-}; // WindowSettings
+	bool fullscreen = false;
+	Window::FullscreenType fstype = Window::FULLSCREEN_DESKTOP;
+	bool vsync = true;
+	int msaa = 0;
+	bool resizable = false;
+	int minwidth = 1;
+	int minheight = 1;
+	bool borderless = false;
+	bool centered = true;
+	int display = 0;
+	bool highdpi = false;
+	double refreshrate = 0.0;
+	bool useposition = false;
+	int x = 0;
+	int y = 0;
+};
 
 } // window
 } // love
